require 'spec_helper'

describe ConfigCore::Parameter do
  let(:value) { 'value' }
  let(:new_value) { 'new value' }
  let(:parameter) { ConfigCore::Parameter.new(:name, value) }

  subject { parameter}

  its(:name) { should be :name }
  its(:value) { should eq value }
  its(:default_value) { should eq value }
  its(:default_value) { should_not be subject.value }
  
  it { expect { parameter.value = new_value }.to_not change { parameter.default_value }.from(value) }

  context 'when editable' do
    its(:editable?) { should be_true }

    it { expect { parameter.value = new_value }.to change { parameter.value }.from(value).to(new_value) }
  end

  context 'when uneditable' do
    before { parameter.disable_editing! }

    its(:editable?) { should be_false }

    it { expect { parameter.value = new_value }.to raise_error(ConfigCore::TryingToModifyUneditableParameterError) }
  end
end
