require 'spec_helper'

describe ConfigCore::Scope do
  let(:scope) { ConfigCore::Scope.new(:core) }
  let(:value) { 'value' }
  let(:new_value) { 'new_value' }

  subject { scope }

  its(:name) { should be :core }

  describe '#empty?' do
    it { expect { scope.define(:items_per_page, 10) }.to change { scope.empty? }.from(true).to(false) }
  end

  describe '#has_parameter?' do
    it { expect { scope.define(:items_per_page, 10) }.to change { scope.has_parameter?(:items_per_page) }.from(false).to(true) }
  end

  describe '#define_parameter' do
    it { expect { scope.define(:param, value) }.to change { scope.has_parameter?(:param) }.to(true) }

    context 'when parameter already exists' do
      before { scope.define(:param, value) }

      it { expect { scope.define(:param, value) }.to raise_error(ConfigCore::ParameterAlreadyExistsError) }
    end

    describe 'parameter/parameter value accessors' do
      context 'when editable parameter' do
        before { scope.define(:param, value) }

        its(:param_parameter) { should be_a ConfigCore::Parameter }
        its(:param) { should == value }
        it { expect { scope.param = new_value }.to change { scope.param }.to(new_value) }
      end

      context 'when uneditable parameter' do
        before { scope.define(:param, value, editable: false) }

        its(:param_parameter) { should be_a ConfigCore::Parameter }
        its(:param) { should == value }
        it { expect { scope.param = new_value }.to raise_error(ConfigCore::TryingToModifyUneditableParameterError) }
      end
    end
  end

end
