require 'spec_helper'

describe ConfigCore::ScopesCollection do
  let!(:scopes_collection) { ConfigCore::ScopesCollection.new }

  subject { scopes_collection }

  describe '#to_h' do
    context 'when empty' do
      its(:to_h) { should == {} }
    end

    context 'when scopes defined' do
      before { scopes_collection.define(:core) }

      it { scopes_collection.to_h.should have(1).scope }
    end
  end

  describe '#[]' do
    context 'when empty' do
      it { scopes_collection[:core].should be_nil }
    end

    context 'when scopes defined' do
      before { scopes_collection.define(:core) }

      it { scopes_collection[:core].should be_a ConfigCore::Scope }
    end
  end

  describe '#has_scope?' do
    it { expect { scopes_collection.define(:core).to change { scopes_collection.has_scope?(:core) }.from(false).to(true) } }
  end

  describe '#define' do
    context 'when scope already exists' do
      before { scopes_collection.define(:core) }

      it { expect { scopes_collection.define(:core) }.to raise_error(ConfigCore::ScopeAlreadyExistsError) }
    end

    context 'when scope not exists' do
      it { expect { scopes_collection.define(:core) }.to change { scopes_collection.has_scope?(:core) }.from(false).to(true)}
      
      describe 'scope reader' do
        before { scopes_collection.define(:core) }

        it { scopes_collection.should respond_to(:core) }
        its(:core) { should be_a ConfigCore::Scope }
      end
    end
  end
end